# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "ca660"


def test_result(self):
    date = datetime.datetime(2010, 3, 28, 13, 50, 0)
    result = self.decode(
        """
        H|\\^&|||CA-600^ 00-22||||||||1
        P|1
        O|1||000001^01^1^B||R||||||N
        R|1|^^^041^PT T^100^1| 10.2|sec ||N||||||20100328135000
        R|2|^^^042^PT%^100^1| 99.4|%||N||||||20100328135000
        R|3|^^^044^PT INR ^100^1| 0.81|-||N||||||20100328135000
        R|4|^^^051^FSL^100^1| 27.4|sec ||N||||||20100328135000
        R|5|^^^061^Fbg^100^1| 8.5|sec ||N||||||20100328135000
        R|6|^^^062^Fbg C. ^100^1| 588|mg/dL||N||||||20100328135000
        L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1",
                "rack": "000001-01",
                "type": "patient",
                "results": [
                    {"code": "041", "value": " 10.2", "date": date, "flags": ["N"]},
                    {"code": "042", "value": " 99.4", "date": date, "flags": ["N"]},
                    {"code": "044", "value": " 0.81", "date": date, "flags": ["N"]},
                    {"code": "051", "value": " 27.4", "date": date, "flags": ["N"]},
                    {"code": "061", "value": " 8.5", "date": date, "flags": ["N"]},
                    {"code": "062", "value": " 588", "date": date, "flags": ["N"]},
                ],
            }
        ],
    }
    assert result == expected


def test_query(self):
    result = self.decode(
        """
        H|\\^&|||CA-600^ 00-22||||||||1
        Q|1|000001^01^      1234^B||^^^040^PT T\\^^^050^FSL\\^^^060^Fbg\\^^^500^+Fbg\\^^^510^-Fbg|0|20100328133300
        L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "1234", "type": "patient"}],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||HostName^^^^|||||CA-600
        P|1
        O|1|^^123456||^^^A^^100\\^^^B^^100|S|20130603110342|||||N
        L|1|N"""
    assert result == expected
