# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.tests import BaseTest


class TestProtocolAlere(BaseTest):
    PROTOCOL_CLASS = "alere"

    HEL_MESSAGE = """
    <?xml version="1.0" encoding="utf-8"?>
    <HEL.R01>
        <HDR>
            <HDR.control_id V="1"/>
            <HDR.version_id V="POCT1"/>
            <HDR.creation_dttm V="2018-05-14T09:28:40+00:00"/>
        </HDR>
        <DEV>
            <DEV.device_id V="1620000170"/>
            <DEV.vendor_id V="ALERE.SCR"/>
            <DEV.serial_id V="1620000170"/>
            <DEV.manufacturer_name V="Alere Scarborough, Inc."/>
            <DEV.hw_version V="priam"/>
            <DEV.sw_version V="v1.3.0-build2"/>
            <DEV.device_name V="alere-reader"/>
            <DCP>
            <DCP.application_timeout V="30"/>
            </DCP>
            <DSC>
            <DSC.connection_profile_cd/>
            <DSC.max_message_sz V="4096"/>
            </DSC>
        </DEV>
    </HEL.R01>
    """

    DST_MESSAGE = """
    <?xml version="1.0" encoding="utf-8"?>
    <DST.R01>
        <HDR>
            <HDR.control_id V="2"/>
            <HDR.version_id V="POCT1"/>
            <HDR.creation_dttm V="2017-02-20T11:27:21-08:00"/>
        </HDR>
        <DST>
            <DST.status_dttm V="2017-02-20T11:27:21-08:00"/>
            <DST.new_observations_qty V="1"/>
            <DST.condition_cd V="B"/>
        </DST>
    </DST.R01>
    """

    OBS_R01_MESSAGE = """
    <?xml version="1.0" encoding="utf-8"?>
    <OBS.R01>
        <HDR>
        <HDR.control_id V="3"/>
        <HDR.version_id V="POCT1"/>
        <HDR.creation_dttm V="2018-05-14T09:28:42+00:00"/>
        </HDR>
        <SVC>
        <SVC.role_cd V="OBS"/>
        <SVC.observation_dttm V="2018-05-14T09:28:24+00:00"/>
        <SVC.status_cd V="NRM"/>
        <SVC.reason_cd V="NEW"/>
        <PT>
            <PT.patient_id V="M10001"/>
            <OBS>
            <OBS.observation_id V="Legionella"/>
            <OBS.value V="negative"/>
            <OBS.method_cd V="M"/>
            <OBS.status_cd V="A"/>
            </OBS>
        </PT>
        <OPR>
            <OPR.operator_id V="1111"/>
        </OPR>
        <RGT>
            <RGT.name V="Legionella"/>
            <RGT.lot_number V="905536"/>
            <RGT.expiration_date V="2018-12-01"/>
        </RGT>
        </SVC>
    </OBS.R01>
    """

    OBS_R02_MESSAGE = """
    <?xml version="1.0" encoding="utf-8"?>
    <OBS.R02>
        <HDR>
            <HDR.control_id V="3"/>
            <HDR.version_id V="POCT1"/>
            <HDR.creation_dttm V="2017-02-21T19:19:34-08:00"/>
        </HDR>
        <SVC>
            <SVC.role_cd V="LQC"/>
            <SVC.observation_dttm V="2017-02-21T14:52:37-08:00"/>
            <SVC.status_cd V="NRM"/>
            <SVC.reason_cd V="NEW"/>
            <CTC>
                <CTC.name V="Control swab"/>
                <CTC.lot_number V="149240"/>
                <CTC.expiration_date V="2050-05-31"/>
                <CTC.level_cd V="pos"/>
                <OBS>
                    <OBS.observation_id V="Flu A" SN="ALERE.SCR"/>
                    <OBS.qualitative_value V="pass"/>
                    <OBS.method_cd V="M"/>
                    <OBS.status_cd V="A"/>
                </OBS>
                <OBS>
                    <OBS.observation_id V="Flu B" SN="ALERE.SCR"/>
                    <OBS.qualitative_value V="fail"/>
                    <OBS.method_cd V="M"/>
                    <OBS.status_cd V="A"/>
                </OBS>
            </CTC>
            <OPR>
                <OPR.operator_id V="291054"/>
            </OPR>
            <RGT>
                <RGT.name V="BinaxNOW Flu A+B"/>
                <RGT.lot_number V="002209902123456"/>
                <RGT.expiration_date V="2099-02-28"/>
            </RGT>
        </SVC>
    </OBS.R02>
    """

    EOT_MESSAGE = """
    <?xml version="1.0" encoding="utf-8"?>
    <EOT.R01>
        <HDR>
            <HDR.control_id V="4"/>
            <HDR.version_id V="POCT1"/>
            <HDR.creation_dttm V="2018-05-14T09:28:45+00:00"/>
        </HDR>
        <EOT>
            <EOT.topic_cd V="OBS"/>
            <EOT.update_dttm V="2018-05-14T09:28:45+00:00"/>
        </EOT>
    </EOT.R01>
    """

    def test_ack_hel(self):
        message = self.P.get_ack_message(
            self.HEL_MESSAGE.encode(), date=datetime.datetime(2018, 5, 14, 9, 28, 42)
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="1"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><ACK><ACK.type_cd V="AA"/><ACK.ack_control_id V="1"/></ACK></ACK.R01>"""
        assert message == expected

    def test_ack_dst(self):
        message = self.P.get_ack_message(
            self.DST_MESSAGE.encode(), date=datetime.datetime(2018, 5, 14, 9, 28, 42)
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="2"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><ACK><ACK.type_cd V="AA"/><ACK.ack_control_id V="2"/></ACK></ACK.R01>"""
        assert message == expected

    def test_req_dst(self):
        message = self.P.get_req_message(
            self.DST_MESSAGE.encode(), date=datetime.datetime(2018, 5, 14, 9, 28, 42)
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><REQ.R01><HDR><HDR.control_id V="3"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><REQ><REQ.request_cd V="ROBS"/></REQ></REQ.R01>"""
        assert message == expected

    def test_ack_obs_r01(self):
        message = self.P.get_ack_message(
            self.OBS_R01_MESSAGE.encode(),
            date=datetime.datetime(2018, 5, 14, 9, 28, 42),
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="4"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><ACK><ACK.type_cd V="AA"/><ACK.ack_control_id V="3"/></ACK></ACK.R01>"""
        assert message == expected

    def test_ack_obs_r02(self):
        message = self.P.get_ack_message(
            self.OBS_R02_MESSAGE.encode(),
            date=datetime.datetime(2018, 5, 14, 9, 28, 42),
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="4"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><ACK><ACK.type_cd V="AA"/><ACK.ack_control_id V="3"/></ACK></ACK.R01>"""
        assert message == expected

    def test_end_eot(self):
        message = self.P.get_end_message(
            self.EOT_MESSAGE.encode(), date=datetime.datetime(2018, 5, 14, 9, 28, 42)
        )
        expected = b"""<?xml version="1.0" encoding="utf-8"?><END.R01><HDR><HDR.control_id V="5"/><HDR.version_id V="POCT1"/><HDR.creation_dttm V="2018-05-14T09:28:42"/></HDR><TRM><TRM.reason_cd V="NRM"/></TRM></END.R01>"""
        assert message == expected

    def test_recv(self):
        self.assertRecvMessageEqual(
            """recv <?xml version="1.0" encoding="utf-8"?><A><HEL.R01><A>    <HDR><A>        <HDR.control_id V="1"/><A>        <HDR.version_id V="POCT1"/><A>        <HDR.creation_dttm V="2018-05-14T09:28:40+00:00"/><A>    </HDR><A>    <DEV><A>        <DEV.device_id V="1620000170"/><A>        <DEV.vendor_id V="ALERE.SCR"/><A>        <DEV.serial_id V="1620000170"/><A>        <DEV.manufacturer_name V="Alere Scarborough, Inc."/><A>        <DEV.hw_version V="priam"/><A>        <DEV.sw_version V="v1.3.0-build2"/><A>        <DEV.device_name V="alere-reader"/><A>        <DCP><A>            <DCP.application_timeout V="30"/><A>        </DCP><A>        <DSC><A>            <DSC.connection_profile_cd/><A>            <DSC.max_message_sz V="4096"/><A>        </DSC><A>    </DEV><A></HEL.R01><A>""",
            self.HEL_MESSAGE,
        )
        self.assertRecvMessageEqual(
            """recv <?xml version="1.0" encoding="utf-8"?><lf><DST.R01><lf>    <HDR><lf>        <HDR.control_id V="2"/><lf>        <HDR.version_id V="POCT1"/><lf>        <HDR.creation_dttm V="2017-02-20T11:27:21-08:00"/><lf>    </HDR><lf>    <DST><lf>        <DST.status_dttm V="2017-02-20T11:27:21-08:00"/><lf>        <DST.new_observations_qty V="1"/><lf>        <DST.condition_cd V="B"/><lf>    </DST><lf></DST.R01><lf>""",
            self.DST_MESSAGE,
        )
        self.assertRecvMessageEqual(
            """recv <?xml version="1.0" encoding="utf-8"?><A><OBS.R01><A>    <HDR><A>        <HDR.control_id V="3"/><A>        <HDR.version_id V="POCT1"/><A>        <HDR.creation_dttm V="2018-05-14T09:28:42+00:00"/><A>    </HDR><A>    <SVC><A>        <SVC.role_cd V="OBS"/><A>        <SVC.observation_dttm V="2018-05-14T09:28:24+00:00"/><A>        <SVC.status_cd V="NRM"/><A>        <SVC.reason_cd V="NEW"/><A>        <PT><A>            <PT.patient_id V="M10001"/><A>            <OBS><A>                <OBS.observation_id V="Legionella"/><A>                <OBS.value V="negative"/><A>                <OBS.method_cd V="M"/><A>                <OBS.status_cd V="A"/><A>            </OBS><A>        </PT><A>        <OPR><A>            <OPR.operator_id V="1111"/><A>        </OPR><A>        <RGT><A>            <RGT.name V="Legionella"/><A>            <RGT.lot_number V="905536"/><A>            <RGT.expiration_date V="2018-12-01"/><A>        </RGT><A>    </SVC><A></OBS.R01><A>""",
            self.OBS_R01_MESSAGE,
        )
        self.assertRecvMessageEqual(
            """recv <?xml version="1.0" encoding="utf-8"?><lf><OBS.R02><lf>    <HDR><lf>            <HDR.control_id V="3"/><lf>            <HDR.version_id V="POCT1"/><lf>            <HDR.creation_dttm V="2017-02-21T19:19:34-08:00"/><lf>    </HDR><lf>    <SVC><lf>            <SVC.role_cd V="LQC"/><lf>            <SVC.observation_dttm V="2017-02-21T14:52:37-08:00"/><lf>            <SVC.status_cd V="NRM"/><lf>            <SVC.reason_cd V="NEW"/><lf>            <CTC><lf>                <CTC.name V="Control swab"/><lf>                <CTC.lot_number V="149240"/><lf>                <CTC.expiration_date V="2050-05-31"/><lf>                <CTC.level_cd V="pos"/><lf>                <OBS><lf>                    <OBS.observation_id V="Flu A" SN="ALERE.SCR"/><lf>                    <OBS.qualitative_value V="pass"/><lf>                    <OBS.method_cd V="M"/><lf>                    <OBS.status_cd V="A"/><lf>                </OBS><lf>                <OBS><lf>                    <OBS.observation_id V="Flu B" SN="ALERE.SCR"/><lf>                    <OBS.qualitative_value V="fail"/><lf>                    <OBS.method_cd V="M"/><lf>                    <OBS.status_cd V="A"/><lf>                </OBS><lf>            </CTC><lf>            <OPR><lf>                <OPR.operator_id V="291054"/><lf>            </OPR><lf>            <RGT><lf>                <RGT.name V="BinaxNOW Flu A+B"/><lf>                <RGT.lot_number V="002209902123456"/><lf>                <RGT.expiration_date V="2099-02-28"/><lf>            </RGT><lf>        </SVC><lf>    </OBS.R02><lf>""",
            self.OBS_R02_MESSAGE,
        )
        self.assertRecvMessageEqual(
            """recv <?xml version="1.0" encoding="utf-8"?><A><EOT.R01><A>    <HDR><A>        <HDR.control_id V="4"/><A>        <HDR.version_id V="POCT1"/><A>        <HDR.creation_dttm V="2018-05-14T09:28:45+00:00"/><A>    </HDR><A>    <EOT><A>        <EOT.topic_cd V="OBS"/><A>        <EOT.update_dttm V="2018-05-14T09:28:45+00:00"/><A>    </EOT><A></EOT.R01><A>""",
            self.EOT_MESSAGE,
        )
