# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "shimadzu"


def test_encode_order(self):
    _order = """DL AQ DP			NYME ANO	123456	0:Unknown						0	10"""
    message = self.order()
    assert message == _order


def test_decode_result_insight(self):
    worklist = self.decode(
        """
ID#	1
Name	24 D
Flags	Flag ID	Data Filename	%RSD (Area)	Conc. (ppb)	Area	Height	Ref 1 Actual Ratio	Accuracy(%)	S/N	Std. Conc. (ppb)	RT
1			J1_gamme 10_005					0,00			10,0000	7,652
2			J1_gamme 10_006					0,00			10,0000	7,652
3			J1_gamme 20_007					0,00			20,0000	7,652
4			J1_gamme 20_008					0,00			20,0000	7,652

ID#	2
Name	245 T
Flags	Flag ID	Data Filename	%RSD (Area)	Conc. (ppb)	Area	Height	Ref 1 Actual Ratio	Accuracy(%)	S/N	Std. Conc. (ppb)	RT
1			J1_gamme 10_005					0,00			10,0000	9,358
2			J1_gamme 10_006					0,00			10,0000	9,358
3			J1_gamme 20_007					0,00			20,0000	9,358
4			J1_gamme 20_008					0,00			20,0000	9,358"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "24 D1Data Filename",
                        "value": "J1_gamme 10_005",
                        "date": None,
                    },
                    {"code": "24 D1Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {"code": "24 D1Std. Conc. (ppb)", "value": "10.0000", "date": None},
                    {"code": "24 D1RT", "value": "7.652", "date": None},
                    {
                        "code": "24 D2Data Filename",
                        "value": "J1_gamme 10_006",
                        "date": None,
                    },
                    {"code": "24 D2Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {"code": "24 D2Std. Conc. (ppb)", "value": "10.0000", "date": None},
                    {"code": "24 D2RT", "value": "7.652", "date": None},
                    {
                        "code": "24 D3Data Filename",
                        "value": "J1_gamme 20_007",
                        "date": None,
                    },
                    {"code": "24 D3Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {"code": "24 D3Std. Conc. (ppb)", "value": "20.0000", "date": None},
                    {"code": "24 D3RT", "value": "7.652", "date": None},
                    {
                        "code": "24 D4Data Filename",
                        "value": "J1_gamme 20_008",
                        "date": None,
                    },
                    {"code": "24 D4Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {"code": "24 D4Std. Conc. (ppb)", "value": "20.0000", "date": None},
                    {"code": "24 D4RT", "value": "7.652", "date": None},
                ],
                "sample_id": "1",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "245 T1Data Filename",
                        "value": "J1_gamme 10_005",
                        "date": None,
                    },
                    {"code": "245 T1Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {
                        "code": "245 T1Std. Conc. (ppb)",
                        "value": "10.0000",
                        "date": None,
                    },
                    {"code": "245 T1RT", "value": "9.358", "date": None},
                    {
                        "code": "245 T2Data Filename",
                        "value": "J1_gamme 10_006",
                        "date": None,
                    },
                    {"code": "245 T2Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {
                        "code": "245 T2Std. Conc. (ppb)",
                        "value": "10.0000",
                        "date": None,
                    },
                    {"code": "245 T2RT", "value": "9.358", "date": None},
                    {
                        "code": "245 T3Data Filename",
                        "value": "J1_gamme 20_007",
                        "date": None,
                    },
                    {"code": "245 T3Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {
                        "code": "245 T3Std. Conc. (ppb)",
                        "value": "20.0000",
                        "date": None,
                    },
                    {"code": "245 T3RT", "value": "9.358", "date": None},
                    {
                        "code": "245 T4Data Filename",
                        "value": "J1_gamme 20_008",
                        "date": None,
                    },
                    {"code": "245 T4Ref 1 Actual Ratio", "value": "0.00", "date": None},
                    {
                        "code": "245 T4Std. Conc. (ppb)",
                        "value": "20.0000",
                        "date": None,
                    },
                    {"code": "245 T4RT", "value": "9.358", "date": None},
                ],
                "sample_id": "2",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected


def test_decode_result_other(self):
    self.configure(shimadzu_analyseur="UPLC1;UPLC2;LC")
    worklist = self.decode(
        """
    Data Filename;Sample Name;Sample ID;Methyl paraben;Ethyl paraben;Propyl paraben;Butyl paraben;
    C:\\LabSolutions\\Sample\\LC\\Demo_Data-001.lcd;STD;1;1,001;1,002;1,001;1,001;
    C:\\LabSolutions\\Sample\\LC\\Demo_Data-002.lcd;STD;2;1,998;1,997;1,999;1,998;
    C:\\LabSolutions\\Sample\\LC\\Demo_Data-003.lcd;STD;3;4,001;4,001;4,000;4,001;
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Methyl paraben",
                        "value": "1.001",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Ethyl paraben",
                        "value": "1.002",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Propyl paraben",
                        "value": "1.001",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Butyl paraben",
                        "value": "1.001",
                        "date": None,
                        "instrument": "LC",
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "Methyl paraben",
                        "value": "1.998",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Ethyl paraben",
                        "value": "1.997",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Propyl paraben",
                        "value": "1.999",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Butyl paraben",
                        "value": "1.998",
                        "date": None,
                        "instrument": "LC",
                    },
                ],
                "sample_id": "2",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "Methyl paraben",
                        "value": "4.001",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Ethyl paraben",
                        "value": "4.001",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Propyl paraben",
                        "value": "4.000",
                        "date": None,
                        "instrument": "LC",
                    },
                    {
                        "code": "Butyl paraben",
                        "value": "4.001",
                        "date": None,
                        "instrument": "LC",
                    },
                ],
                "sample_id": "3",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(shimadzu_qc_prefix="QC")
    date = datetime.datetime.now()
    date = date.replace(second=0, microsecond=0)
    worklist = self.decode(
        """
    Data Filename;Sample Name;Sample ID;VITB1;VITB6;
    C:\\LabSolutions\\Sample\\LC\\Demo_Data-003 - VITB1B6.lcd; NIVEAU 1 probatoire;QC_VB1VB6_LEV1;4,004;4,010;
    C:\\LabSolutions\\Sample\\LC\\Demo_Data-004 - VITB1B6.lcd;niveau 2 prop;QC_VB1VB6_LEV2;3,018;3,020;
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "VITB1", "date": date, "value": "4.004", "instrument": ""},
                    {"code": "VITB6", "date": date, "value": "4.010", "instrument": ""},
                ],
                "control_lot": "QC_VB1VB6_LEV1",
                "sample_id": "QC_VB1VB6_LEV1",
                "type": "control",
            },
            {
                "results": [
                    {"code": "VITB1", "date": date, "value": "3.018", "instrument": ""},
                    {"code": "VITB6", "date": date, "value": "3.020", "instrument": ""},
                ],
                "control_lot": "QC_VB1VB6_LEV2",
                "sample_id": "QC_VB1VB6_LEV2",
                "type": "control",
            },
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    Data Filename;Sample Name;Sample ID;Cilostazol;Cilostazol-d11;OPC-13015;OPC-13217;
    C:\\SPAS\\Files\\LogData0180880807_1415_00000090_01_S_cilo2.lcd;;20180807_Request-0090;10.000;10.000;10.000;10.000;
    C:\\SPAS\\Files\\LogData0180880807_1415_00000091_02_S_cilo2.lcd;;20180807_Request-0091;100.000;100.000;100.000;100.000;
    C:\\SPAS\\Files\\LogData0180880806_0957_00000052_01_N_RESERP.lcd;;20180806_Request-0052;-----;-----;-----;-----;
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Cilostazol",
                        "value": "10.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "Cilostazol-d11",
                        "value": "10.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13015",
                        "value": "10.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13217",
                        "value": "10.000",
                        "date": None,
                        "instrument": "",
                    },
                ],
                "sample_id": "20180807_Request-0090",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "Cilostazol",
                        "value": "100.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "Cilostazol-d11",
                        "value": "100.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13015",
                        "value": "100.000",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13217",
                        "value": "100.000",
                        "date": None,
                        "instrument": "",
                    },
                ],
                "sample_id": "20180807_Request-0091",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "Cilostazol",
                        "value": "-----",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "Cilostazol-d11",
                        "value": "-----",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13015",
                        "value": "-----",
                        "date": None,
                        "instrument": "",
                    },
                    {
                        "code": "OPC-13217",
                        "value": "-----",
                        "date": None,
                        "instrument": "",
                    },
                ],
                "sample_id": "20180806_Request-0052",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
