# -*- coding: utf8 -*-
from __future__ import unicode_literals

parser_class = "wasp"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||LIS|||||WASP||P|1|20130715141912
    P|1|036666666|L036666666||NYME^ANO||20110513|F||||||||1.70|58
    O|1|123456||^^^A\\^^^B|A||20130603110342||||N||||Serum||||||||||Q
    O|2|123456||^^^C|A||20130603110342||||C||||Serum||||||||||Q
    L|1|F
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|1|WASP|3|4|5|6|7|8|P|1394-97|20161223143427
    Q|1|18875819||ALL||||||||O
    L|1|F
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "18875819"}],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
        H|\\^&|||WASP|||||LIS||P|20161223143446
        O|1|18875818||||||
        R|1|18875818||||||N|
        O|2|18875819||||||N|
        R|2|18875819||||||X|
        L|1
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "18875818",
                "results": [{"code": "WASP", "value": "N"}],
            },
            {"type": "patient", "sample_id": "18875819"},
        ],
    }
    assert worklist == expected


def test_decode_result_copan(self):
    worklist = self.decode(
        """
    H|\\^&|||WASP|||||LIS||P|20170303155541
    O|1|01110296293||||||
    R|5|PUS|COS||1005436050||20170303155508|N|
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "01110296293",
                "results": [
                    {"code": "WASP", "value": "N"},
                    {"code": "PUS_GELOSE", "value": "COS"},
                    {"code": "PUS_LOT", "value": "1005436050"},
                    {"code": "PUS_STREAKING", "value": "20170303155508"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_other_status(self):
    self.configure(wasp_other_status=True)
    worklist = self.decode(
        """
        H|\\^&|||WASP|||||LIS||P|20161223143446
        O|1|18875818||||||
        R|1|18875818||||||N|
        O|2|18875819||||||N|
        R|2|18875819||||||X|
        O|3|18875820||||||N|
        R|3|18875820||||||R|
        L|1
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "18875818",
                "results": [{"code": "WASP", "value": "N"}],
            },
            {
                "type": "patient",
                "sample_id": "18875819",
                "results": [{"code": "WASP", "value": "X"}],
            },
            {
                "type": "patient",
                "sample_id": "18875820",
                "results": [{"code": "WASP", "value": "R"}],
            },
        ],
    }
    assert worklist == expected
