# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "wasplab"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||LIS|||||WASPLAB||||20130715141912
    P|1||036666666||ANO^NYME||20110513|F|||||1||||||||||||LOC
    O|1|123456||^^^C|S||20130603110342||||||||Serum|||||CHOC^PVX|01^01
    R|1|1^^I^1
    R|2|2^^I^2
    R|3|||A^42^A
    R|4|||B^201302131516^A
    L|1
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&||appkey|WASPLAB-READING|||||LIS
    Q|1|11006020||||||||LOGIN|S|D
    L|1|F
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "11006020"}],
    }
    assert worklist == expected


def test_decode_result_boite(self):
    date = datetime.datetime(2019, 1, 16, 15, 6, 49)
    worklist = self.decode(
        """
    H|\\^&||$appkey|wasp585|||||LIS||P|1|20190116150649
    P|1|
    O|1|478589313314||||||
    R|1|PV|478589313314|0h^S|T|||P||wasp585|20190116150420|20190116150649|
    R|2|PV|478589313314^01^|LAME^^^S^|M|^|||||20190116150420|20190116150649|
    R|3|PV|478589313314^02^|PVX4^1006960190^2019-03-21^S^|M|^|||||20190116150420|20190116150649|
    R|4|PV|478589313314^03^|ANCA3^1006959630^2019-02-06^S^|M|^|||||20190116150420|20190116150649|
    R|5|PV|478589313314^04^|CAN^1007001310^2019-01-16^S^|M|^|||||20190116150420|20190116150649|
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "478589313314",
                "results": [
                    {
                        "code": "PV_TIME",
                        "date": date,
                        "instrument": "wasp585",
                        "value": "0h",
                    },
                    {
                        "code": "PV",
                        "date": date,
                        "instrument": "wasp585",
                        "value": "0h",
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 1, 16, 15, 6, 49),
                        "instrument": "",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_0h",
                        "date": date,
                        "flags": ["S"],
                        "instrument": "",
                        "value": "LAME",
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 1, 16, 15, 6, 49),
                        "instrument": "",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_0h",
                        "date": date,
                        "flags": ["S"],
                        "instrument": "",
                        "value": "PVX4",
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 1, 16, 15, 6, 49),
                        "instrument": "",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_0h",
                        "date": date,
                        "flags": ["S"],
                        "instrument": "",
                        "value": "ANCA3",
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 1, 16, 15, 6, 49),
                        "instrument": "",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_0h",
                        "date": date,
                        "flags": ["S"],
                        "instrument": "",
                        "value": "CAN",
                    },
                    {
                        "code": "PVBOITE_0h",
                        "value": "LAME;PVX4;ANCA3;CAN",
                        "lots": [
                            {"lot": "1006960190", "type": "reagent"},
                            {"lot": "1006959630", "type": "reagent"},
                            {"lot": "1007001310", "type": "reagent"},
                        ],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2016, 7, 28, 19, 23, 45)
    worklist = self.decode(
        """
    H|\\^&|||WASPLAB-READING|||||LIS
    P|1|
    O|1|11006020||||||
    R|1|CBU^01^|11006020|48h^P|T|NEG48^\\CAM5||F||WASP487|20160728095023|20160728192345|
    R|2|CBU^01|201709130003^01^|CPSE^ggg^2016-01-15^S|M||||||20170913164359|20170913165041|
    R|3|CBU^01^|11006020^01^1|NI^|G|^|
    R|3|CBU^01|201709300002^01^1|NUMER^EG105^|G|^|||||20170929110111|20171002164432|
    R|5|CBU^02^|11006020^02^2|ECOL^COUNT|G|^|
    R|6|CBU^02^|11006020^02^2|MYLA^|W|^|
    R|7|CBU^03^|11006020^03^|NI^|G|^|
    R|8|CBU^04^|11006020^04^3|CAMP^|G|^|
    R|9|CBU^01^|11006020^04^3|MYLA^|W|^|
    L|1|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "11006020",
                "results": [
                    {
                        "code": "CBU_TIME",
                        "value": "48h",
                        "instrument": "WASP487",
                        "date": date,
                    },
                    {
                        "code": "CBU",
                        "value": "48h",
                        "instrument": "WASP487",
                        "date": date,
                    },
                    {
                        "code": "CBU_SAMPLE",
                        "value": "NEG48",
                        "instrument": "WASP487",
                        "date": date,
                    },
                    {"code": "CBU_SAMPLE2", "value": "NEG48;CAM5", "date": date},
                    {
                        "code": "CBU_STATUS",
                        "date": datetime.datetime(2017, 9, 13, 16, 50, 41),
                        "instrument": "",
                        "value": "S",
                    },
                    {
                        "code": "CBUENS_48h",
                        "value": "CPSE",
                        "flags": ["S"],
                        "instrument": "",
                        "date": datetime.datetime(2017, 9, 13, 16, 50, 41),
                    },
                    {"code": "CBU_G1", "value": "NI", "instrument": "", "date": None},
                    {"code": "CBU_CPSE", "value": "NI", "instrument": "", "date": None},
                    {
                        "code": "CBU_G1",
                        "value": "NUMER",
                        "instrument": "",
                        "date": datetime.datetime(2017, 10, 2, 16, 44, 32),
                    },
                    {
                        "code": "CBU_CPSE",
                        "value": "NUMER",
                        "instrument": "",
                        "date": datetime.datetime(2017, 10, 2, 16, 44, 32),
                    },
                    {
                        "code": "CBU_NUMER",
                        "value": "EG105",
                        "instrument": "",
                        "date": datetime.datetime(2017, 10, 2, 16, 44, 32),
                    },
                    {"code": "CBU_G2", "value": "ECOL", "instrument": "", "date": None},
                    {
                        "code": "CBU_CPSE",
                        "value": "ECOL",
                        "instrument": "",
                        "date": None,
                    },
                    {"code": "CBU_W2", "value": "MYLA", "instrument": "", "date": None},
                    {"code": "CBU_G", "value": "NI", "instrument": "", "date": None},
                    {"code": "CBU_CPSE", "value": "NI", "instrument": "", "date": None},
                    {"code": "CBU_G3", "value": "CAMP", "instrument": "", "date": None},
                    {
                        "code": "CBU_CPSE",
                        "value": "CAMP",
                        "instrument": "",
                        "date": None,
                    },
                    {"code": "CBU_W3", "value": "MYLA", "instrument": "", "date": None},
                    {
                        "code": "CBUBOITE_48h",
                        "value": "CPSE",
                        "lots": [{"lot": "ggg", "type": "reagent"}],
                    },
                    {"code": "CBU_WTASK", "value": "MYLA"},
                    {
                        "code": "IDENT_CBU",
                        "value": {
                            "germes": [
                                {"id": "NI", "num": "", "rang": 1},
                                {"id": "ECOL", "num": "COUNT", "rang": 2},
                                {"id": "CAMP", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                    {
                        "code": "IDENT2_CBU",
                        "value": {
                            "germes": [
                                {"des": "", "id": "NI", "num": "", "rang": 1},
                                {"des": "", "id": "ECOL", "num": "COUNT", "rang": 2},
                                {"des": "", "id": "CAMP", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                    {
                        "code": "IDENT3_CBU",
                        "value": {
                            "germes": [
                                {"des": "", "num": "", "rang": 1},
                                {"des": "", "num": "COUNT", "rang": 2},
                                {"des": "", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_option_ignore_ident_germes(self):
    self.configure(wasplab_ident_ignore_germes="G2,G3")

    worklist = self.decode(
        """
    H|\\^&|||WASPLAB-READING|||||LIS
    P|1|
    O|1|11006020||||||
    R|2|CBU^01^|11006020^01^1|G1^|G|^|
    R|3|CBU^03^|11006020^03^|G2^|G|^|
    R|3|CBU^02^|11006020^02^2|G3^5|G|^|
    R|4|CBU^04^|11006020^04^3|CAMP^|G|^|
    L|1|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "11006020",
                "results": [
                    {"instrument": "", "date": None, "code": "CBU_G1", "value": "G1"},
                    {"instrument": "", "date": None, "code": "CBU_G", "value": "G2"},
                    {"instrument": "", "date": None, "code": "CBU_G2", "value": "G3"},
                    {"instrument": "", "date": None, "code": "CBU_G3", "value": "CAMP"},
                    {
                        "code": "IDENT_CBU",
                        "value": {
                            "germes": [
                                {"id": "G1", "num": "", "rang": 1},
                                {"id": "", "num": "5", "rang": 2},
                                {"id": "CAMP", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                    {
                        "code": "IDENT2_CBU",
                        "value": {
                            "germes": [
                                {"des": "", "id": "G1", "num": "", "rang": 1},
                                {"des": "G3", "id": "", "num": "5", "rang": 2},
                                {"des": "", "id": "CAMP", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                    {
                        "code": "IDENT3_CBU",
                        "value": {
                            "germes": [
                                {"des": "", "num": "", "rang": 1},
                                {"des": "G3", "num": "5", "rang": 2},
                                {"des": "", "num": "", "rang": 3},
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": None,
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_wasp_hors_media(self):
    self.configure(wasplab_hors_media=True)
    worklist = self.decode(
        """
    H|\\^&|||WASP|||||LIS||P|20170303155541
    O|1|01110296293||||||
    R|5|PUS|COS||1005436050||20170303155508|N|
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "01110296293",
                "results": [
                    {"code": "WASP", "value": "N"},
                    {"code": "PUS_GELOSE", "value": "COS"},
                    {"code": "PUS_LOT", "value": "1005436050"},
                    {"code": "PUS_STREAKING", "value": "20170303155508"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_wasp_champ_r7_multiple(self):
    self.configure(wasplab_hors_media=True)
    worklist = self.decode(
        """
    H|\\^&|||WASP|||||LIS||P|20170303155541
    O|1|01110296293||||||
    R|1|ECBU|9201PP90924001|18h^P|T|BABSSTRB^\\ECOL5^||F||eng
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "01110296293",
                "results": [
                    {"code": "WASP", "value": "F"},
                    {"code": "ECBU_GELOSE", "value": "9201PP90924001"},
                    {"code": "ECBU_LOT", "value": "T"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_wasp_hors_media_version2(self):
    self.configure(wasplab_hors_media=True)
    worklist = self.decode(
        """
    H|\\^&|||WASP|||||LIS||P|20190719110958
    O|1|049071901006|||1007357780|||
    R|1|^^^CBU|CPSE|||||N|
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "049071901006",
                "results": [
                    {"code": "WASP", "value": "N"},
                    {"code": "CBU_GELOSE", "value": "CPSE"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_rank_germ_zero(self):
    date = datetime.datetime(2019, 11, 25, 14, 18, 58)
    worklist = self.decode(
        """
    H|\\^&||$appkey|WASPLAB-READING|||||LIS||P|1|20191125141858
    P|1|
    O|1|25111902||||||
    R|1|PV|25111902|1h^P|T|BAVIS^\\BCONS^\\^||P||labadmin|20191125134443|20191125141858|
    R|2|PV|25111902^01^|PVX^1007661690^2020-02-06^S^0h|M|^||P||labadmin|20191125134443|20191125141858|
    R|3|PV|25111902^01^1|GERPUR^RARES^|G|^||||labadmin|20191125134443|20191125141858|
    R|4|PV|25111902^01^1|IDENT^|W|||||labadmin|20191125134443|20191125141858|
    R|5|PV|25111902^02^|CNA^1007688100^2020-01-07^S^0h|M|^||P||labadmin|20191125134443|20191125141858|
    R|6|PV|25111902^02^0|BCONS^^|G|^||||labadmin|20191125134443|20191125141858|
    R|7|PV|25111902^03^|CAN2^1007627760^2020-01-13^S^0h|M|^||P||labadmin|20191125134443|20191125141858|
    R|8|PV|25111902^03^0|BAVIS^^|G|^||||labadmin|20191125134443|20191125141858|
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "PV_TIME",
                        "instrument": "labadmin",
                        "value": "1h",
                        "date": date,
                    },
                    {
                        "code": "PV",
                        "instrument": "labadmin",
                        "value": "1h",
                        "date": date,
                    },
                    {
                        "code": "PV_SAMPLE",
                        "instrument": "labadmin",
                        "value": "BAVIS",
                        "date": date,
                    },
                    {"code": "PV_SAMPLE2", "value": "BAVIS;BCONS", "date": date},
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 11, 25, 14, 18, 58),
                        "instrument": "labadmin",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_1h",
                        "flags": ["S"],
                        "instrument": "labadmin",
                        "value": "PVX",
                        "date": date,
                    },
                    {
                        "code": "PV_G1",
                        "instrument": "labadmin",
                        "value": "GERPUR",
                        "date": date,
                    },
                    {
                        "code": "PV_PVX",
                        "instrument": "labadmin",
                        "value": "GERPUR",
                        "date": date,
                    },
                    {
                        "code": "PV_W1",
                        "instrument": "labadmin",
                        "value": "IDENT",
                        "date": date,
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 11, 25, 14, 18, 58),
                        "instrument": "labadmin",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_1h",
                        "flags": ["S"],
                        "instrument": "labadmin",
                        "value": "CNA",
                        "date": date,
                    },
                    {
                        "code": "PV_G0",
                        "instrument": "labadmin",
                        "value": "BCONS",
                        "date": date,
                    },
                    {
                        "code": "PV_CNA",
                        "instrument": "labadmin",
                        "value": "BCONS",
                        "date": date,
                    },
                    {
                        "code": "PV_STATUS",
                        "date": datetime.datetime(2019, 11, 25, 14, 18, 58),
                        "instrument": "labadmin",
                        "value": "S",
                    },
                    {
                        "code": "PVENS_1h",
                        "flags": ["S"],
                        "instrument": "labadmin",
                        "value": "CAN2",
                        "date": date,
                    },
                    {
                        "code": "PV_G0",
                        "instrument": "labadmin",
                        "value": "BAVIS",
                        "date": date,
                    },
                    {
                        "code": "PV_CAN2",
                        "instrument": "labadmin",
                        "value": "BAVIS",
                        "date": date,
                    },
                    {
                        "code": "PVBOITE_1h",
                        "lots": [
                            {"lot": "1007661690", "type": "reagent"},
                            {"lot": "1007688100", "type": "reagent"},
                            {"lot": "1007627760", "type": "reagent"},
                        ],
                        "value": "PVX;CNA;CAN2",
                    },
                    {"code": "PV_WTASK", "value": "IDENT"},
                    {
                        "code": "IDENT_PV",
                        "value": {
                            "germes": [{"id": "GERPUR", "num": "RARES", "rang": 1}],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": date,
                    },
                    {
                        "code": "IDENT2_PV",
                        "value": {
                            "germes": [
                                {"des": "", "id": "GERPUR", "num": "RARES", "rang": 1}
                            ],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": date,
                    },
                    {
                        "code": "IDENT3_PV",
                        "value": {
                            "germes": [{"des": "", "num": "RARES", "rang": 1}],
                            "seq": "0",
                            "type_resultat": "identification_germes",
                        },
                        "date": date,
                    },
                ],
                "sample_id": "25111902",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
